/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *  Copyright (C) 2003 Hiroyuki Ikezoe
 *  Copyright (C) 2003 Takuro Ashie
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id: kz-history.c,v 1.17 2004/07/23 14:34:47 ikezoe Exp $
 */

#include "kz-history.h"

#include "intl.h"
#include "gobject-utils.h"
#include "kz-mozembed.h"
#include "estsearch.h"
#include "utils.h"

static void kz_history_class_init (KzHistoryClass *klass);
static void kz_history_init       (KzHistory      *history);
static void kz_history_dispose    (GObject        *object);

static void cb_embed_load_finished (KzMozEmbed *kzembed, KzHistory *history);

static KzBookmarkClass *parent_class = NULL;


KZ_OBJECT_GET_TYPE(kz_history, "KzHistory", KzHistory,
		   kz_history_class_init, kz_history_init,
		   KZ_TYPE_BOOKMARK)


static void
kz_history_class_init (KzHistoryClass *klass)
{
	GObjectClass *object_class;

	parent_class = g_type_class_peek_parent (klass);
	object_class = (GObjectClass *) klass;

	object_class->dispose      = kz_history_dispose;
#if 0
	object_class->set_property = kz_history_set_property;
	object_class->get_property = kz_history_get_property;
#endif
}


static void
kz_history_init (KzHistory *history)
{
}


static void
kz_history_dispose (GObject *object)
{
	KzHistory *history;
	gchar *history_dir;
	guint limit_days = 30;
	gboolean limit = FALSE;
	time_t limit_seconds;

	g_return_if_fail(KZ_IS_HISTORY(object));

	history = KZ_HISTORY(object);

	KZ_CONF_GET("History", "limit_days", limit_days, INT);
	KZ_CONF_GET("History", "limit_cache", limit, BOOL);
	if (limit)
	{
		limit_seconds = limit_days * 86400;
		history_dir = g_strconcat(g_get_home_dir(), HISTORY_DIR, NULL); 
		purge_history_file(history_dir, limit_seconds);
		g_free(history_dir);
		/* optimize and purge history index */
		if (exists_estindex)
			estsearch_purge_index(NULL);
	}

	if (G_OBJECT_CLASS (parent_class)->dispose)
		G_OBJECT_CLASS (parent_class)->dispose(object);
}


KzHistory *
kz_history_new (const gchar *file)
{
	GObject *obj;
	KzBookmark *bookmark;

	obj = g_object_new(KZ_TYPE_HISTORY,
			   "type",      KZ_BOOKMARK_PURE_FOLDER,
			   "location",  file,
			   "title",     _("History"),
			   "file-type", "XBEL",
			   NULL);
	bookmark = KZ_BOOKMARK(obj);
	bookmark->flags |= KZ_BOOKMARK_FILE_FLAG;

	return KZ_HISTORY(bookmark);
}


void
kz_history_connect_to_embed (KzHistory *history, gpointer *embed)
{
	g_return_if_fail(KZ_IS_HISTORY(history));
	g_return_if_fail(KZ_IS_MOZ_EMBED(embed));
	
	g_signal_connect(G_OBJECT(embed), "net_stop",
			  G_CALLBACK(cb_embed_load_finished), history);
}


void
kz_history_disconnect_to_embed (KzHistory *history, gpointer *embed)
{
	g_return_if_fail(KZ_IS_HISTORY(history));
	g_return_if_fail(KZ_IS_MOZ_EMBED(embed));

	g_signal_handlers_disconnect_by_func(G_OBJECT(embed),
					     G_CALLBACK(cb_embed_load_finished),
					     history);
}


static void
cb_embed_load_finished (KzMozEmbed *kzembed, KzHistory *history)
{
	const gchar *location;
	gchar *filename;
	gchar *cache;

	g_return_if_fail(KZ_IS_MOZ_EMBED(kzembed));
	g_return_if_fail(KZ_IS_HISTORY(history));

	if (!exists_estindex) return;

	cache = g_strconcat("file://", g_get_home_dir(),
			    HISTORY_DIR, NULL);

	location = kz_moz_embed_get_location(kzembed);
	if (!location || 
	    g_str_has_prefix(location, "history-search:") ||
	    g_str_has_prefix(location, cache))
	{
		g_free(cache);
		return;
	}

	filename = kz_moz_embed_store_history_file(kzembed);

	if (!filename) return;

	g_idle_add(estsearch_update_index, filename);
}


